package blobovnicza

import (
	"fmt"
	"strconv"

	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-lens/internal/schema/common"
	cid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id"
	objectSDK "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object"
	oid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object/id"
	"github.com/davecgh/go-spew/spew"
	"github.com/gdamore/tcell/v2"
	"github.com/rivo/tview"
)

type (
	MetaBucket struct{}

	MetaRecord struct {
		label string
		count uint64
	}

	Bucket struct {
		size int64
	}

	Record struct {
		addr   oid.Address
		object objectSDK.Object
	}
)

func (b *MetaBucket) String() string {
	return common.FormatSimple("META", tcell.ColorLime)
}

func (b *MetaBucket) DetailedString() string {
	return spew.Sdump(*b)
}

func (b *MetaBucket) Filter(string, any) common.FilterResult {
	return common.No
}

func (r *MetaRecord) String() string {
	return fmt.Sprintf("%-11s %c %d", r.label, tview.Borders.Vertical, r.count)
}

func (r *MetaRecord) DetailedString() string {
	return spew.Sdump(*r)
}

func (r *MetaRecord) Filter(string, any) common.FilterResult {
	return common.No
}

func (b *Bucket) String() string {
	return common.FormatSimple(strconv.FormatInt(b.size, 10), tcell.ColorLime)
}

func (b *Bucket) DetailedString() string {
	return spew.Sdump(*b)
}

func (b *Bucket) Filter(typ string, _ any) common.FilterResult {
	switch typ {
	case "cid":
		return common.Maybe
	case "oid":
		return common.Maybe
	default:
		return common.No
	}
}

func (r *Record) String() string {
	return fmt.Sprintf(
		"CID %s OID %s %c Object {...}",
		common.FormatSimple(fmt.Sprintf("%-44s", r.addr.Container()), tcell.ColorAqua),
		common.FormatSimple(fmt.Sprintf("%-44s", r.addr.Object()), tcell.ColorAqua),
		tview.Borders.Vertical,
	)
}

func (r *Record) DetailedString() string {
	return spew.Sdump(*r)
}

func (r *Record) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "cid":
		id := val.(cid.ID)
		return common.IfThenElse(r.addr.Container().Equals(id), common.Yes, common.No)
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.addr.Object().Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}
