package records

import (
	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-lens/internal/schema/common"
	cid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id"
	oid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object/id"
)

func (r *GraveyardRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "cid":
		id := val.(cid.ID)
		return common.IfThenElse(r.object.Container().Equals(id), common.Yes, common.No)
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.object.Object().Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *GarbageRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "cid":
		id := val.(cid.ID)
		return common.IfThenElse(r.addr.Container().Equals(id), common.Yes, common.No)
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.addr.Object().Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *ContainerVolumeRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "cid":
		id := val.(cid.ID)
		return common.IfThenElse(r.id.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *ShardInfoRecord) Filter(string, any) common.FilterResult {
	return common.No
}

func (r *LockedRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.id.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *ObjectRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.id.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *SmallRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.id.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *RootRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.id.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *OwnerRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.id.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *UserAttributeRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.id.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *PayloadHashRecord) Filter(string, any) common.FilterResult {
	return common.No
}

func (r *ParentRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.parent.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *SplitRecord) Filter(string, any) common.FilterResult {
	return common.No
}

func (r *ContainerCountersRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "cid":
		id := val.(cid.ID)
		return common.IfThenElse(r.id.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}

func (r *ECInfoRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.id.Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}
