package records

import (
	"git.frostfs.info/TrueCloudLab/frostfs-sdk-go/checksum"
	cid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id"
	objectSDK "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object"
	oid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object/id"
	"github.com/google/uuid"
)

type (
	GraveyardRecord struct {
		object, tombstone oid.Address
	}

	GarbageRecord struct {
		addr oid.Address
	}

	ContainerVolumeRecord struct {
		id     cid.ID
		volume uint64
	}

	LockedRecord struct {
		id  oid.ID
		ids []oid.ID
	}

	ShardInfoRecord struct {
		label string
		value string
	}

	ObjectRecord struct {
		id     oid.ID
		object objectSDK.Object
	}

	SmallRecord struct {
		id        oid.ID
		storageID *string // optional
	}

	RootRecord struct {
		id   oid.ID
		info *objectSDK.SplitInfo // optional
	}

	OwnerRecord struct {
		id oid.ID
	}

	UserAttributeRecord struct {
		id oid.ID
	}

	PayloadHashRecord struct {
		checksum checksum.Checksum
		ids      []oid.ID
	}

	ParentRecord struct {
		parent oid.ID
		ids    []oid.ID
	}

	SplitRecord struct {
		id  uuid.UUID
		ids []oid.ID
	}

	ContainerCountersRecord struct {
		id                      cid.ID
		logical, physical, user uint64
	}

	ECInfoRecord struct {
		id  oid.ID
		ids []oid.ID
	}
)
