package writecache

import (
	"fmt"

	"git.frostfs.info/TrueCloudLab/frostfs-node/cmd/frostfs-lens/internal/schema/common"
	cid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/container/id"
	oid "git.frostfs.info/TrueCloudLab/frostfs-sdk-go/object/id"
	"github.com/davecgh/go-spew/spew"
	"github.com/gdamore/tcell/v2"
	"github.com/rivo/tview"
)

type (
	DefaultBucket struct{}

	DefaultRecord struct {
		addr oid.Address
		data []byte
	}
)

func (b *DefaultBucket) String() string {
	return common.FormatSimple("0 Default", tcell.ColorLime)
}

func (r *DefaultRecord) String() string {
	return fmt.Sprintf(
		"CID %s OID %s %c Data {...}",
		common.FormatSimple(fmt.Sprintf("%-44s", r.addr.Container()), tcell.ColorAqua),
		common.FormatSimple(fmt.Sprintf("%-44s", r.addr.Object()), tcell.ColorAqua),
		tview.Borders.Vertical,
	)
}

func (b *DefaultBucket) DetailedString() string {
	return spew.Sdump(*b)
}

func (r *DefaultRecord) DetailedString() string {
	return spew.Sdump(*r)
}

func (b *DefaultBucket) Filter(typ string, _ any) common.FilterResult {
	switch typ {
	case "cid":
		return common.Maybe
	case "oid":
		return common.Maybe
	default:
		return common.No
	}
}

func (r *DefaultRecord) Filter(typ string, val any) common.FilterResult {
	switch typ {
	case "cid":
		id := val.(cid.ID)
		return common.IfThenElse(r.addr.Container().Equals(id), common.Yes, common.No)
	case "oid":
		id := val.(oid.ID)
		return common.IfThenElse(r.addr.Object().Equals(id), common.Yes, common.No)
	default:
		return common.No
	}
}
